/**
 * Copyright 2007 Steven Van Impe, Department Of Applied Mathematics And Computer Science, Ghent University
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package gramschmidt;

import java.text.NumberFormat;
import java.util.ResourceBundle;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.vecmath.Vector3d;

/**
 * Frame that allows input of initial vectors and provides numerical results.
 *
 * @author Steven Van Impe (steven.vanimpe@ugent.be)
 */
public class GSVectorFrame extends javax.swing.JFrame {
    
    // data
    GSDataModel data;
    // state
    GSStateModel state;
    
    /** Creates new form GSVectorFrame */
    public GSVectorFrame(GSDataModel data, GSStateModel state) {
        this.data = data;
        this.state = state;
        
        initComponents();
    }
    
    /**
     * Make sure the displayed values match the current data.
     * This discards any unsaved input.
     *
     * Call this before making the frame visible.
     */
    public void updateValues() {
        jFormattedTextField1.setValue(data.getVector(1).x);
        jFormattedTextField2.setValue(data.getVector(1).y);
        jFormattedTextField3.setValue(data.getVector(1).z);
        jFormattedTextField4.setValue(data.getVector(2).x);
        jFormattedTextField5.setValue(data.getVector(2).y);
        jFormattedTextField6.setValue(data.getVector(2).z);
        jFormattedTextField7.setValue(data.getVector(3).x);
        jFormattedTextField9.setValue(data.getVector(3).y);
        jFormattedTextField10.setValue(data.getVector(3).z);
        
        jLabel15.setText(String.format("u1: ( %.3f , %.3f , %.3f )", data.getOrthogonalVector(1).x, data.getOrthogonalVector(1).y, data.getOrthogonalVector(1).z));
        jLabel16.setText(String.format("u2: ( %.3f , %.3f , %.3f )", data.getOrthogonalVector(2).x, data.getOrthogonalVector(2).y, data.getOrthogonalVector(2).z));
        jLabel17.setText(String.format("u3: ( %.3f , %.3f , %.3f )", data.getOrthogonalVector(3).x, data.getOrthogonalVector(3).y, data.getOrthogonalVector(3).z));
        jLabel19.setText(String.format("e1: ( %.3f , %.3f , %.3f )", data.getNormalizedVector(1).x, data.getNormalizedVector(1).y, data.getNormalizedVector(1).z));
        jLabel20.setText(String.format("e2: ( %.3f , %.3f , %.3f )", data.getNormalizedVector(2).x, data.getNormalizedVector(2).y, data.getNormalizedVector(2).z));
        jLabel21.setText(String.format("e3: ( %.3f , %.3f , %.3f )", data.getNormalizedVector(3).x, data.getNormalizedVector(3).y, data.getNormalizedVector(3).z));
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jFormattedTextField1 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jLabel3 = new javax.swing.JLabel();
        jFormattedTextField2 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jLabel4 = new javax.swing.JLabel();
        jFormattedTextField3 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jFormattedTextField4 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jFormattedTextField5 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jFormattedTextField6 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        jLabel13 = new javax.swing.JLabel();
        jFormattedTextField7 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jFormattedTextField9 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jFormattedTextField10 = new JFormattedTextField(NumberFormat.getNumberInstance());
        jLabel11 = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jLabel14 = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        jLabel16 = new javax.swing.JLabel();
        jLabel17 = new javax.swing.JLabel();
        jLabel18 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        jLabel20 = new javax.swing.JLabel();
        jLabel21 = new javax.swing.JLabel();

        setTitle(java.util.ResourceBundle.getBundle("properties/gramschmidt").getString("vectorsWindowTitle"));
        setResizable(false);
        jLabel1.setText(java.util.ResourceBundle.getBundle("properties/gramschmidt").getString("vectorsLabel"));

        jLabel2.setText("v1: (");

        jFormattedTextField1.setColumns(3);
        jFormattedTextField1.setText("jFormattedTextField1");

        jLabel3.setText(",");

        jFormattedTextField2.setColumns(3);
        jFormattedTextField2.setText("jFormattedTextField2");

        jLabel4.setText(",");

        jFormattedTextField3.setColumns(3);
        jFormattedTextField3.setText("jFormattedTextField3");

        jLabel5.setText(")");

        jLabel6.setText("v2: (");

        jFormattedTextField4.setColumns(3);
        jFormattedTextField4.setText("jFormattedTextField4");

        jFormattedTextField5.setColumns(3);
        jFormattedTextField5.setText("jFormattedTextField5");

        jFormattedTextField6.setColumns(3);
        jFormattedTextField6.setText("jFormattedTextField6");

        jLabel7.setText(",");

        jLabel8.setText(",");

        jLabel9.setText(")");

        jLabel10.setText("v3: (");

        jLabel12.setText(",");

        jLabel13.setText(")");

        jFormattedTextField7.setColumns(3);
        jFormattedTextField7.setText("jFormattedTextField7");

        jFormattedTextField9.setColumns(3);
        jFormattedTextField9.setText("jFormattedTextField9");

        jFormattedTextField10.setColumns(3);
        jFormattedTextField10.setText("jFormattedTextField10");

        jLabel11.setText(",");

        jButton1.setText(java.util.ResourceBundle.getBundle("properties/gramschmidt").getString("applyButton"));
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jLabel14.setText(java.util.ResourceBundle.getBundle("properties/gramschmidt").getString("orthogonalVectorsLabel"));

        jLabel15.setText("jLabel15");

        jLabel16.setText("jLabel16");

        jLabel17.setText("jLabel17");

        jLabel18.setText(java.util.ResourceBundle.getBundle("properties/gramschmidt").getString("normalizedVectorsLabel"));

        jLabel19.setText("jLabel19");

        jLabel20.setText("jLabel20");

        jLabel21.setText("jLabel21");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel1)
                    .add(layout.createSequentialGroup()
                        .add(12, 12, 12)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(jLabel6)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jFormattedTextField4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel7)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jFormattedTextField5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel8)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jFormattedTextField6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel9))
                            .add(layout.createSequentialGroup()
                                .add(jLabel2)
                                .add(12, 12, 12)
                                .add(jFormattedTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel3)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jFormattedTextField2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel4)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jFormattedTextField3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel5))
                            .add(layout.createSequentialGroup()
                                .add(jLabel10)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jFormattedTextField7, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel11)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jFormattedTextField9, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel12)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jFormattedTextField10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel13)))))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap(141, Short.MAX_VALUE)
                .add(jButton1)
                .addContainerGap())
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel14)
                    .add(layout.createSequentialGroup()
                        .add(12, 12, 12)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel16)
                            .add(jLabel15)
                            .add(jLabel17))))
                .addContainerGap(108, Short.MAX_VALUE))
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel18)
                    .add(layout.createSequentialGroup()
                        .add(12, 12, 12)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel20)
                            .add(jLabel19)
                            .add(jLabel21))))
                .addContainerGap(81, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(jFormattedTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel3)
                    .add(jFormattedTextField2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel4)
                    .add(jFormattedTextField3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel5))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel6)
                    .add(jFormattedTextField4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jFormattedTextField6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel8)
                    .add(jLabel9)
                    .add(jLabel7)
                    .add(jFormattedTextField5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel10)
                    .add(jFormattedTextField7, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel11)
                    .add(jFormattedTextField9, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel12)
                    .add(jLabel13)
                    .add(jFormattedTextField10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jButton1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel14)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel15)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel16)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel17)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel18)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel19)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel20)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel21)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents
    
    // try to update the data model with the supplied vectors
    // the model will reject vectors of length 0
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        Vector3d v1 = new Vector3d(((Number) jFormattedTextField1.getValue()).doubleValue(), ((Number) jFormattedTextField2.getValue()).doubleValue(), ((Number) jFormattedTextField3.getValue()).doubleValue());
        Vector3d v2 = new Vector3d(((Number) jFormattedTextField4.getValue()).doubleValue(), ((Number) jFormattedTextField5.getValue()).doubleValue(), ((Number) jFormattedTextField6.getValue()).doubleValue());
        Vector3d v3 = new Vector3d(((Number) jFormattedTextField7.getValue()).doubleValue(), ((Number) jFormattedTextField9.getValue()).doubleValue(), ((Number) jFormattedTextField10.getValue()).doubleValue());
        
        if (! data.initialize(v1, v2, v3))
            JOptionPane.showMessageDialog(this, ResourceBundle.getBundle("properties/gramschmidt").getString("invalidVectorsMessage"), ResourceBundle.getBundle("properties/gramschmidt").getString("invalidVectorsTitle"), JOptionPane.ERROR_MESSAGE);
        else {
            state.setState(GSStateModel.STATE_INITIALIZED);
            updateValues();
        }
    }//GEN-LAST:event_jButton1ActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JFormattedTextField jFormattedTextField1;
    private javax.swing.JFormattedTextField jFormattedTextField10;
    private javax.swing.JFormattedTextField jFormattedTextField2;
    private javax.swing.JFormattedTextField jFormattedTextField3;
    private javax.swing.JFormattedTextField jFormattedTextField4;
    private javax.swing.JFormattedTextField jFormattedTextField5;
    private javax.swing.JFormattedTextField jFormattedTextField6;
    private javax.swing.JFormattedTextField jFormattedTextField7;
    private javax.swing.JFormattedTextField jFormattedTextField9;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel21;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    // End of variables declaration//GEN-END:variables

}
